/*
 *  OpenPiano - a MIDI-based piano trainer with an easy note presentation
 *  Copyright (C) 2008  Sascha Westermann
 *
 *  This program is free software; you can redistribute it and/or modify it 
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *  for more details.
 *  
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

package openpiano.gl;

import com.sun.opengl.util.FPSAnimator;
import javax.media.opengl.GL;
import javax.media.opengl.GLCanvas;
import javax.media.opengl.GLEventListener;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.glu.GLU;
import openpiano.control.Controller;
import openpiano.settings.Constants;

/**
 * The GLCore class represents the graphical main loop
 * 
 * @author Sascha Westermann
 * @version 1.0, 07/30/08
 */

public class GLCore extends GLCanvas implements GLEventListener {
	private static final long serialVersionUID = 6211076206610435832L;
	private Controller controller;
	private GL gl;
	private GLU glu;
	private FPSAnimator animator;

	/**
	 * Constructs a new GL core
	 * 
     * @param controller the main controller of the application
	 */
    public GLCore(Controller controller) {
		super();
    	this.controller = controller;
	}

    /**
     * Initializes the GL settings
     * 
     * @param drawable the GL context to deal with
     */
    public void init(GLAutoDrawable drawable) {
		gl = drawable.getGL();
		glu = new GLU();
		gl.setSwapInterval(1);
		
		controller.initPiano(gl);
		
		float l_position[] = {0.0f, 60.0f, 60.0f, 1.0f};
		gl.glEnable(GL.GL_LIGHTING);
		gl.glEnable(GL.GL_LIGHT0);
		gl.glEnable(GL.GL_DEPTH_TEST);
		gl.glEnable(GL.GL_NORMALIZE);
		gl.glLightfv(GL.GL_LIGHT0, GL.GL_POSITION, l_position, 0);
		gl.glEnable(GL.GL_TEXTURE_2D);
		gl.glShadeModel(GL.GL_SMOOTH);
		gl.glClearColor(0.7f, 0.7f, 0.7f, 0.0f);
		
		animator = new FPSAnimator(drawable, Constants.FPS, true);
		animator.start();
	}

    /**
     * Draws all items, the main GL loop
     * 
     * @param drawable the GL context to deal with
     */
    public void display(GLAutoDrawable drawable) {
    	gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
		gl.glClearColor(0.7f, 0.7f, 0.7f, 0.0f);

		gl.glMatrixMode(GL.GL_PROJECTION);
		gl.glLoadIdentity();
		glu.gluPerspective(60.0f,(float)drawable.getWidth()/(float)drawable.getHeight(),1.0,3000);
		gl.glMatrixMode(GL.GL_MODELVIEW);
		gl.glLoadIdentity();
		glu.gluLookAt( 0.0f, 10.0f, 37.0f, 0.0f, 10.0f, 0.0f, 0.0f, 1.0f, 0.0f);
				
		
		gl.glPushMatrix();
		gl.glRotatef(controller.getSettings().getAngle(), 1.0f, 0.0f, 0.0f);
		gl.glTranslatef(0.0f, 0.0f, 9.0f);
		gl.glScalef(controller.getSettings().getZoomFactor(), controller.getSettings().getZoomFactor(), controller.getSettings().getZoomFactor());
		gl.glTranslatef(0.0f, 0.0f, -9.0f);

		showScene();
		gl.glPopMatrix();
	}

    /**
     * Registers a changed display
     * 
     * @param drawable the GL context to deal with
     * @param modeChanged <tt>true</tt> if mode changed
     * @param deviceChanged <tt>true</tt> if device changed
     */
    public void displayChanged(GLAutoDrawable drawable, boolean modeChanged, boolean deviceChanged) {}

    /**
     * Registers a reshape of the application window
     * 
     * @param drawable the GL context to deal with
     * @param x the new x position
     * @param y the new y position
     * @param w the new window width
     * @param h the new window height
     */
    public void reshape(GLAutoDrawable drawable, int x, int y, int w, int h) {
    }
    
    private void showScene() {
		gl.glPushMatrix();
		gl.glTranslatef(-controller.getCamera().getXPosition(), 0.0f, 0.0f);
		if (controller.getMidiLoader() != null)
			controller.getNotes().showNotes();
		controller.getPiano().showPiano();
		controller.getNotes().showLines();
		gl.glPopMatrix();
    }
}
