/*
 *  OpenPiano - a MIDI-based piano trainer with an easy note presentation
 *  Copyright (C) 2008  Sascha Westermann
 *
 *  This program is free software; you can redistribute it and/or modify it 
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *  for more details.
 *  
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

package openpiano.midi;

import javax.sound.midi.MetaMessage;

/**
 * The Bpm class consists time-based beats per minutes events
 * 
 * @author Sascha Westermann
 * @version 1.0, 07/30/08
 */

public class Bpm{
	private int bpm;
	private long tick;

	/**
	 * Constructs a new beats per minute event
	 * 
	 * @param metaMessage the MIDI message containing informations about beats per minute
	 * @param tick the start tick to handle the MIDI message
	 */
	public Bpm(MetaMessage metaMessage, long tick) {
		this.tick = tick;
		byte[] metaData = metaMessage.getData();
		long microBpm = ((metaData[0] & 0xFF) << 16) 
			| ((metaData[1] & 0xFF) << 8) | (metaData[2] & 0xFF);
		
		if (microBpm <= 0)
			microBpm = 1;
		bpm = (int)(Math.round(60000000.0f/microBpm));
	}

	/**
	 * Returns the start tick of the beats per minute event
	 * 
	 * @return the start tick of the beats per minute event
	 */
	public long getTick() { return tick; }
	
	/**
	 * Returns the beats per minute
	 * 
	 * @return the beats per minute
	 */
	public int getBpm() { return bpm; };
}