/*
 *  OpenPiano - a MIDI-based piano trainer with an easy note presentation
 *  Copyright (C) 2008  Sascha Westermann
 *
 *  This program is free software; you can redistribute it and/or modify it 
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *  for more details.
 *  
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

package openpiano.midi;

import javax.sound.midi.MidiMessage;

/**
 * The MidiControl class consists time-based MIDI control messages
 * 
 * @author Sascha Westermann
 * @version 1.0, 07/30/08
 */

public class MidiControl implements Comparable<MidiControl> {
	long tick;
	private MidiMessage midiMessage;

	/**
	 * Constructs a new MIDI control message
	 * 
	 * @param midiMessage the MIDI control message
	 * @param tick the start tick of the MIDI message
	 */
	public MidiControl(MidiMessage midiMessage, long tick) {
		this.tick = tick;
		this.midiMessage = midiMessage;
	}
	
	/**
	 * Compares the ticks of MIDI control messages
	 * 
	 * @param midiControl the MIDI control message to compare with
	 * @return the difference of the ticks
	 */
	public int compareTo(MidiControl midiControl) {
		return (int)(this.tick - midiControl.getTick());
	}

	/**
	 * Returns the MIDI control message
	 *
	 * @return the MIDI control message
	 */
	public MidiMessage getMidiMessage() { return midiMessage; }
	
	/**
	 * Returns the start tick of the MIDI control message
	 * 
	 * @return the start tick of the MIDI control message
	 */
	public long getTick() { return tick; }
}
