/*
 *  OpenPiano - a MIDI-based piano trainer with an easy note presentation
 *  Copyright (C) 2008  Sascha Westermann
 *
 *  This program is free software; you can redistribute it and/or modify it 
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *  for more details.
 *  
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

package openpiano.midi;

import javax.sound.midi.MidiDevice;
import javax.sound.midi.MidiMessage;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.MidiUnavailableException;
import javax.sound.midi.Receiver;
import javax.sound.midi.ShortMessage;
import javax.sound.midi.Transmitter;
import openpiano.control.Controller;

/**
 * The MidiInput class receives events from a selected MIDI in port
 * 
 * @author Sascha Westermann
 * @version 1.0, 07/30/08
 */

public class MidiInput implements Receiver {
	private MidiDevice midiKeyboard;
	private Controller controller;
	private Transmitter transmitter;

	/**
	 * Constructs a new Receiver from a MIDI in port
	 * 
     * @param controller the main controller of the application
	 * @param midiDevice the MIDI in port
	 */
	public MidiInput(Controller controller, MidiDevice.Info midiDevice) {
		this.controller = controller;
		
		try {
			midiKeyboard = MidiSystem.getMidiDevice(midiDevice);
			midiKeyboard.open();
		} catch (MidiUnavailableException e) { System.out.println("MIDI keyboard not available"); }

		try	{
			transmitter = midiKeyboard.getTransmitter();
			transmitter.setReceiver(this);
		} catch (MidiUnavailableException e) {
			System.out.println("cannot connect the MIDI keyboard to the MidiKeyboardHandler");
		}
	}

	/**
	 * Closes the receiver
	 */
	public void close() {
		transmitter.close();
		midiKeyboard.close();
	}

	/**
	 * Sends a MIDI command to the main controller
	 * 
	 * @param midiMessage the MIDI command
	 * @param timeStamp the time stamp of the MIDI command
	 */
	public void send(MidiMessage midiMessage, long timeStamp) {
		if (midiMessage instanceof ShortMessage) {
			ShortMessage sMessage = (ShortMessage)midiMessage;
			controller.send(sMessage);
		}
	}
}
