/*
 *  OpenPiano - a MIDI-based piano trainer with an easy note presentation
 *  Copyright (C) 2008  Sascha Westermann
 *
 *  This program is free software; you can redistribute it and/or modify it 
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *  for more details.
 *  
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

package openpiano.midi;

import javax.sound.midi.MetaMessage;
import javax.sound.midi.MidiEvent; 
import javax.sound.midi.MidiMessage; 
import javax.sound.midi.MidiSystem; 
import javax.sound.midi.Sequence; 
import javax.sound.midi.ShortMessage; 
import javax.sound.midi.Track; 
import java.io.File;

public class Recorder {
	private int bpm, tpb, tps;
	private File outputFile;
	private Boolean recording = false;
	private Sequence sequence;
	private long startTime;
	private Track track;
	
	public Recorder(int bpm, int tpb) {
		this.bpm = bpm;
		this.tpb = tpb;
		tps = Math.round((bpm/60.0f) * tpb);
		
		String home = System.getProperty("user.home");
		String separator = System.getProperty("file.separator");
		outputFile = new File(home + separator + "output.mid");
	}

	public void addMessage(MidiMessage midiMessage) {
		long currentTime = System.currentTimeMillis();
		long diff = currentTime - startTime;
		long tick = Math.round(tps * (diff/1000.0d));
		track.add(new MidiEvent(midiMessage, tick));
	}
	
	private byte[] bpmToByte(int bpm) {
		int microBpm = Math.round(60000000.0f/(float)bpm);

		byte[] result = new byte[3];
		result[0] = (byte)((microBpm >> 16) & 0xFF);
		result[1] = (byte)((microBpm >> 8)  & 0x00FF);
		result[2] = (byte)( microBpm        & 0x0000FF);
		
		return result;
	}
	
	public Boolean isRecording() { return recording; }
	
	public void startRecording() {
		try {
			sequence = new Sequence(Sequence.PPQ, tpb);
		} catch (Exception e) {}
		
		track = sequence.createTrack();
		
		try {
			// set beats per minute
			MetaMessage setBPM = new MetaMessage();
			setBPM.setMessage(81, bpmToByte(bpm), 3);
			
			// set instrument to acoustic grand piano
			ShortMessage setInstrument = new ShortMessage();
			setInstrument.setMessage(ShortMessage.PROGRAM_CHANGE, 0, 0);

			// set volume to max
			ShortMessage setVolume = new ShortMessage();
			setVolume.setMessage(ShortMessage.CONTROL_CHANGE, 7, 127);
			
			track.add(new MidiEvent(setBPM, 0));
			track.add(new MidiEvent(setInstrument, 0));
			track.add(new MidiEvent(setVolume, 0));
		} catch (Exception e) {}
		
		startTime = System.currentTimeMillis();
		recording = true;
	}
	
	public void stopRecording() {
		recording = false;
		
		try {
			MidiSystem.write(sequence, 0, outputFile);
		} catch (Exception e) {}
	}
}
